<?php

namespace FiloBlu\Flow\Model\Channel\In;

use Exception;

/**
 * Class Stock
 * @package FiloBlu\Flow\Model\Channel\In
 */
class Stock extends AbstractModel
{

    protected $lookupConversion = null;
    protected $connection;

    /**
     * @param $data
     * @return bool
     * @throws Exception
     */
    public function insertData($data)
    {
        $parser = $this->getParser();

        if ($useMap = $this->getChannelConfig()->usesMap()) {
            if (!isset($this->map) || empty($this->map)) {
                throw  new Exception('Stock has option \'use map = yes\' but no map is provided');
            }

            $this->map = $parser->map;
        }

        $check_mapping_count = $parser->checkMappingCount();

        $flow = $this->getFile()->getData('flow');
        $model = $this->objectManager->create("FiloBlu\Flow\Model\\{$flow}");

        if (!$model) {
            return false;
        }

        $model->setData('meta_file', $this->getFile()->getId());
        $model->setData('meta_processed', 0);

        foreach ($data as $field => $value) {

            // Have a json mapping?
            if ($useMap) {

                // Get the mapping
                $fieldMap = $this->map->get($field);
                // I need to skip this?
                if (!isset($fieldMap['skip'])) {
                    // Have a destination field? Set that as a 'to field'
                    if (isset($fieldMap['to']) && $fieldMap['to']) {
                        $to_field = $fieldMap['to'];
                        $model->setData($to_field, $value);
                    // Don't have a destination field but the check_mapping_count is off? Fallback to column name
                    } elseif (!$check_mapping_count) {
                        $model->setData($field, $value);
                    }
                }

                // No json mapping here
            } else {
                $model->setData($field, $value);
            }

        }

        // SKU Lookup
        $lookup_value = $model->getData(\FiloBlu\Flow\Block\Adminhtml\Channel\Edit\Tab\Config::SKU_CONVERSION_LOOKUP_FIELD);
        if($lookup_value){
            $sku = $this->lookupSku($lookup_value);
            if ($sku){
                $model->setData('sku',$sku);
            }
        }

        if (!$model->getData('magazzino')) {
            $stock_source = $this->getConfig('stock_source');
            if($stock_source){
                $model->setData('magazzino', $stock_source);
            }else{
                $model->setData('magazzino', 'default');
            }
        }

        $model->save();
    }

    /**
     * @return array|null
     */
    public function getSkuConversionLookup(){
        $a=1;
        if ($this->lookupConversion){
            return $this->lookupConversion;
        }
        $attribute_code = $this->getConfig('sku_conversion_lookup');
        $catalog_product_type_id = 4;
        $sql = "SELECT CPE.sku,CPEV.value AS 'lookup' FROM catalog_product_entity CPE
                LEFT JOIN catalog_product_entity_varchar CPEV ON CPE.row_id = CPEV.row_id AND CPEV.attribute_id IN (
                    SELECT attribute_id 
                    FROM eav_attribute 
                    WHERE attribute_code = '{$attribute_code}' AND entity_type_id = {$catalog_product_type_id}
                    )
                WHERE CPE.type_id IN ('simple','bundle','virtual')
                AND CPEV.value IS NOT NULL";
        $result = $this->getConnection()->fetchAll($sql);
        $lookup = [];
        // Parse the result
        foreach ($result as $row => $item) {
            $lookup[$item['lookup']] = $item['sku'];
        }
        // Save it
        $this->lookupConversion = $lookup;
        return $lookup;
    }

    /**
     * @param $needle
     * @return mixed|null
     */
    private function lookupSku($needle){
        $attribute_code = $this->getConfig('sku_conversion_lookup');
        if ($attribute_code == \FiloBlu\Flow\Block\Adminhtml\Channel\Edit\Tab\Config::SKU_CONVERSION_LOOKUP_DISABLED || !$attribute_code){
            return $needle;
        }
        
        $haystack = $this->getSkuConversionLookup();
        if (isset($haystack[$needle])){
            return $haystack[$needle];
        }
        return $needle;
    }

    protected function getConnection()
    {
        if (!$this->connection) {
            $resource = $this->objectManager->create('\Magento\Framework\App\ResourceConnection');
            $this->connection = $resource->getConnection('core_write');
        }
        return $this->connection;
    }


}
